"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.logSameRangeMismatch = exports.logUnsupportedMismatch = exports.logMissingSnappedToMismatch = exports.logMissingLocalVersion = exports.pipeline = exports.listMismatches = void 0;
const chalk_1 = __importDefault(require("chalk"));
const effect_1 = require("effect");
const os_1 = require("os");
const tag_1 = require("../config/tag");
const constants_1 = require("../constants");
const default_error_handlers_1 = require("../error-handlers/default-error-handlers");
const get_context_1 = require("../get-context");
const get_instances_1 = require("../get-instances");
const io_1 = require("../io");
const exit_if_invalid_1 = require("../io/exit-if-invalid");
const get_group_header_1 = require("../lib/get-group-header");
const pad_start_1 = require("../lib/pad-start");
const with_logger_1 = require("../lib/with-logger");
function listMismatches({ io, cli, errorHandlers = default_error_handlers_1.defaultErrorHandlers }) {
    return (0, effect_1.pipe)((0, get_context_1.getContext)({ io, cli, errorHandlers }), effect_1.Effect.flatMap((ctx) => pipeline(ctx, io, errorHandlers)), effect_1.Effect.flatMap(exit_if_invalid_1.exitIfInvalid), effect_1.Effect.provide((0, effect_1.pipe)(effect_1.Context.empty(), effect_1.Context.add(tag_1.CliConfigTag, cli), effect_1.Context.add(io_1.IoTag, io))), with_logger_1.withLogger);
}
exports.listMismatches = listMismatches;
/** Exported to be reused by `syncpack lint` */
function pipeline(ctx, io, errorHandlers) {
    return effect_1.Effect.gen(function* ($) {
        const { versionGroups } = yield* $((0, get_instances_1.getInstances)(ctx, io, errorHandlers));
        let index = 0;
        for (const group of versionGroups) {
            const groupSize = group.instances.length;
            let fixableCount = 0;
            let unfixableCount = 0;
            let validCount = 0;
            if (group._tag === 'FilteredOut') {
                index++;
                continue;
            }
            yield* $(effect_1.Effect.logInfo((0, get_group_header_1.getVersionGroupHeader)({ group, index })));
            if (group._tag === 'Ignored') {
                const msg = (0, chalk_1.default) `{gray ${(0, pad_start_1.padStart)(groupSize)} ${constants_1.ICON.rightArrow} ignored}`;
                yield* $(effect_1.Effect.logInfo(msg));
                index++;
                continue;
            }
            for (const groupReport of yield* $(group.inspectAll())) {
                for (const report of groupReport.reports) {
                    const _tag = report._tag;
                    if (_tag === 'Valid') {
                        validCount++;
                    }
                    else if (_tag === 'Banned') {
                        ctx.isInvalid = true;
                        yield* $(logBanned(report));
                        fixableCount++;
                    }
                    else if (_tag === 'HighestSemverMismatch') {
                        ctx.isInvalid = true;
                        yield* $(logHighestSemverMismatch(report));
                        fixableCount++;
                    }
                    else if (_tag === 'LocalPackageMismatch') {
                        ctx.isInvalid = true;
                        yield* $(logLocalPackageMismatch(report));
                        fixableCount++;
                    }
                    else if (_tag === 'LowestSemverMismatch') {
                        ctx.isInvalid = true;
                        yield* $(logLowestSemverMismatch(report));
                        fixableCount++;
                    }
                    else if (_tag === 'PinnedMismatch') {
                        ctx.isInvalid = true;
                        yield* $(logPinnedMismatch(report));
                        fixableCount++;
                    }
                    else if (_tag === 'SemverRangeMismatch') {
                        ctx.isInvalid = true;
                        yield* $(logSemverRangeMismatch(report));
                        fixableCount++;
                    }
                    else if (_tag === 'SnappedToMismatch') {
                        ctx.isInvalid = true;
                        yield* $(logSnappedToMismatch(report));
                        fixableCount++;
                    }
                    else if (_tag === 'MissingLocalVersion') {
                        ctx.isInvalid = true;
                        yield* $(logMissingLocalVersion(report));
                        unfixableCount++;
                    }
                    else if (_tag === 'MissingSnappedToMismatch') {
                        ctx.isInvalid = true;
                        yield* $(logMissingSnappedToMismatch(report));
                        unfixableCount++;
                    }
                    else if (_tag === 'UnsupportedMismatch') {
                        ctx.isInvalid = true;
                        yield* $(logUnsupportedMismatch(report));
                        unfixableCount++;
                    }
                    else if (_tag === 'SameRangeMismatch') {
                        ctx.isInvalid = true;
                        yield* $(logSameRangeMismatch(report));
                        unfixableCount++;
                    }
                }
            }
            if (validCount)
                yield* $(logValidSize(validCount));
            if (fixableCount)
                yield* $(logFixableSize(fixableCount));
            if (unfixableCount)
                yield* $(logUnfixableSize(unfixableCount));
            index++;
        }
        return ctx;
    });
}
exports.pipeline = pipeline;
function logValidSize(amount) {
    const msg = (0, chalk_1.default) `${(0, pad_start_1.padStart)(amount)} {green ${constants_1.ICON.tick}} already valid`;
    return effect_1.Effect.logInfo(msg);
}
function logFixableSize(amount) {
    const msg = (0, chalk_1.default) `${(0, pad_start_1.padStart)(amount)} {green ${constants_1.ICON.tick}} can be auto-fixed`;
    return effect_1.Effect.logInfo(msg);
}
function logUnfixableSize(amount) {
    const msg = (0, chalk_1.default) `{red ${(0, pad_start_1.padStart)(amount)} ${constants_1.ICON.panic} can be fixed manually using} {blue syncpack prompt}`;
    return effect_1.Effect.logInfo(msg);
}
function logBanned(report) {
    const _tag = report._tag;
    const instance = report.fixable.instance;
    const name = instance.name;
    const jsonFile = instance.packageJsonFile.jsonFile;
    const path = instance.strategy.path;
    const shortPath = jsonFile.shortPath;
    return effect_1.Effect.logInfo((0, chalk_1.default) `{red ${constants_1.ICON.cross}} ${name} {red banned} {gray ${shortPath} > ${path}} {blue [${_tag}]}`);
}
function logHighestSemverMismatch(report) {
    const _tag = report._tag;
    const fixable = report.fixable;
    const instance = fixable.instance;
    const jsonFile = instance.packageJsonFile.jsonFile;
    const actual = instance.rawSpecifier;
    const expected = fixable.raw;
    const name = instance.name;
    const path = instance.strategy.path;
    const shortPath = jsonFile.shortPath;
    return effect_1.Effect.logInfo((0, chalk_1.default) `{red ${constants_1.ICON.cross}} ${name} {red ${actual}} {dim ${constants_1.ICON.rightArrow}} {green ${expected}} {gray ${shortPath} > ${path}} {blue [${_tag}]}`);
}
function logLocalPackageMismatch(report) {
    const _tag = report._tag;
    const fixable = report.fixable;
    const instance = fixable.instance;
    const actual = instance.rawSpecifier;
    const expected = fixable.raw;
    const name = instance.name;
    const path = instance.strategy.path;
    const shortPath = instance.packageJsonFile.jsonFile.shortPath;
    return effect_1.Effect.logInfo((0, chalk_1.default) `{red ${constants_1.ICON.cross}} ${name} {red ${actual}} {dim ${constants_1.ICON.rightArrow}} {green ${expected}} {gray ${shortPath} > ${path}} {blue [${_tag}]}`);
}
function logLowestSemverMismatch(report) {
    const _tag = report._tag;
    const fixable = report.fixable;
    const instance = fixable.instance;
    const actual = instance.rawSpecifier;
    const expected = fixable.raw;
    const name = instance.name;
    const path = instance.strategy.path;
    const shortPath = instance.packageJsonFile.jsonFile.shortPath;
    return effect_1.Effect.logInfo((0, chalk_1.default) `{red ${constants_1.ICON.cross}} ${name} {red ${actual}} {dim ${constants_1.ICON.rightArrow}} {green ${expected}} {gray ${shortPath} > ${path}} {blue [${_tag}]}`);
}
function logPinnedMismatch(report) {
    const _tag = report._tag;
    const fixable = report.fixable;
    const instance = fixable.instance;
    const actual = instance.rawSpecifier;
    const expected = fixable.raw;
    const name = instance.name;
    const path = instance.strategy.path;
    const shortPath = instance.packageJsonFile.jsonFile.shortPath;
    return effect_1.Effect.logInfo((0, chalk_1.default) `{red ${constants_1.ICON.cross}} ${name} {red ${actual}} {dim ${constants_1.ICON.rightArrow}} {green ${expected}} {gray ${shortPath} > ${path}} {blue [${_tag}]}`);
}
function logSemverRangeMismatch(report) {
    const _tag = report._tag;
    const fixable = report.fixable;
    const instance = fixable.instance;
    const actual = instance.rawSpecifier;
    const expected = fixable.raw;
    const name = instance.name;
    const path = instance.strategy.path;
    const shortPath = instance.packageJsonFile.jsonFile.shortPath;
    return effect_1.Effect.logInfo((0, chalk_1.default) `{red ${constants_1.ICON.cross}} ${name} {red ${actual}} {dim ${constants_1.ICON.rightArrow}} {green ${expected}} {gray ${shortPath} > ${path}} {blue [${_tag}]}`);
}
function logSnappedToMismatch(report) {
    const _tag = report._tag;
    const fixable = report.fixable;
    const instance = fixable.instance;
    const actual = instance.rawSpecifier;
    const expected = fixable.raw;
    const name = instance.name;
    const path = instance.strategy.path;
    const shortPath = instance.packageJsonFile.jsonFile.shortPath;
    return effect_1.Effect.logInfo((0, chalk_1.default) `{red ${constants_1.ICON.cross}} ${name} {red ${actual}} {dim ${constants_1.ICON.rightArrow}} {green ${expected}} {gray ${shortPath} > ${path}} {blue [${_tag}]}`);
}
function logMissingLocalVersion(report) {
    const instance = report.unfixable;
    const localPath = report.localInstance.packageJsonFile.jsonFile.shortPath;
    const jsonFile = instance.packageJsonFile.jsonFile;
    const actual = instance.rawSpecifier;
    const name = instance.name;
    const path = instance.strategy.path;
    const shortPath = jsonFile.shortPath;
    return effect_1.Effect.logInfo([
        (0, chalk_1.default) `{red ${constants_1.ICON.cross}} ${name} {red ${actual}} {dim ${constants_1.ICON.rightArrow}} {red ???} {gray ${shortPath} > ${path}} {blue [MissingLocalVersion]}`,
        (0, chalk_1.default) `  {red ${localPath} does not have a .version property which is exact semver}`,
    ].join(os_1.EOL));
}
exports.logMissingLocalVersion = logMissingLocalVersion;
function logMissingSnappedToMismatch(report) {
    const instance = report.unfixable;
    const jsonFile = instance.packageJsonFile.jsonFile;
    const actual = instance.rawSpecifier;
    const name = instance.name;
    const path = instance.strategy.path;
    const shortPath = jsonFile.shortPath;
    return effect_1.Effect.logInfo([
        (0, chalk_1.default) `{red ${constants_1.ICON.cross}} ${name} {red ${actual}} {dim ${constants_1.ICON.rightArrow}} {red ???} {gray ${shortPath} > ${path}} {blue [MissingSnappedToMismatch]}`,
        (0, chalk_1.default) `  {red no package in this groups .snapTo array depend on ${name}}`,
    ].join(os_1.EOL));
}
exports.logMissingSnappedToMismatch = logMissingSnappedToMismatch;
function logUnsupportedMismatch(report) {
    const instance = report.unfixable;
    const jsonFile = instance.packageJsonFile.jsonFile;
    const actual = instance.rawSpecifier;
    const name = instance.name;
    const path = instance.strategy.path;
    const shortPath = jsonFile.shortPath;
    return effect_1.Effect.logInfo([
        (0, chalk_1.default) `{red ${constants_1.ICON.cross}} ${name} {red ${actual}} {dim ${constants_1.ICON.rightArrow}} {red ???} {gray ${shortPath} > ${path}} {blue [UnsupportedMismatch]}`,
        (0, chalk_1.default) `  {red use {blue syncpack prompt} to fix this manually}`,
    ].join(os_1.EOL));
}
exports.logUnsupportedMismatch = logUnsupportedMismatch;
function logSameRangeMismatch(report) {
    const instance = report.unfixable;
    const jsonFile = instance.packageJsonFile.jsonFile;
    const actual = instance.rawSpecifier;
    const name = instance.name;
    const path = instance.strategy.path;
    const shortPath = jsonFile.shortPath;
    const mismatches = report.mismatches;
    return effect_1.Effect.logInfo([
        (0, chalk_1.default) `{red ${constants_1.ICON.cross}} ${name} {red range ${actual} does not include ${mismatches.join(', ')}} {gray ${shortPath} > ${path}} {blue [SameRangeMismatch]}`,
        (0, chalk_1.default) `  {gray use {blue syncpack prompt} to fix this manually}`,
    ].join(os_1.EOL));
}
exports.logSameRangeMismatch = logSameRangeMismatch;
